import { relationList } from './field.js';
import { extractDirectedRelation } from './helpers.js';
import { afterDocumentDelete } from './hooks.js';
export const biDirectionalRelationship = (config)=>{
    const { here, there } = extractDirectedRelation(config, 'A->B');
    return {
        config,
        anchors: {
            [here.collection]: relationList(config, 'A->B'),
            [there.collection]: relationList(config, 'B->A')
        },
        hooks: {
            [here.collection]: afterDocumentDelete(config, 'A->B'),
            [there.collection]: afterDocumentDelete(config, 'B->A')
        }
    };
};
export const biDirectionalRelationships = (relationships)=>(incomingConfig)=>({
            ...incomingConfig,
            collections: (incomingConfig.collections || []).map((collection)=>({
                    ...collection,
                    hooks: {
                        ...collection.hooks,
                        afterDelete: [
                            ...collection.hooks?.afterDelete ?? [],
                            ...relationships.filter((relationship)=>Object.keys(relationship.hooks).includes(collection.slug)).map((relationship)=>relationship.hooks[collection.slug])
                        ]
                    }
                })),
            i18n: {
                ...incomingConfig.i18n,
                translations: {
                    de: {
                        ...incomingConfig.i18n?.translations?.de ?? {},
                        biDirectionalRelationships: {
                            validationError: 'Es gibt bereits eine Beziehung zu diesem Element. Duplikate sind nicht erlaubt.'
                        }
                    },
                    en: {
                        ...incomingConfig.i18n?.translations?.en ?? {},
                        biDirectionalRelationships: {
                            validationError: 'There is already a relation to this element. Duplicates are not allowed.'
                        }
                    }
                }
            }
        });

//# sourceMappingURL=index.js.map